@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #ifndef __ACTIONS_H__
@@ -26,19 +26,35 @@
 
 #include <gp-params.h>
 
+enum wait_type {
+	WAIT_TIME,
+	WAIT_EVENTS,
+	WAIT_FRAMES,
+};
+enum download_type { DT_NO_DOWNLOAD, DT_DOWNLOAD };
+struct waitparams {
+	union {
+		int milliseconds;
+		int events;
+		int frames;
+	} u;
+	enum wait_type type;
+	enum download_type downloadtype;
+};
+
 /* Image actions */
-typedef int FileAction    (GPParams *, const char *filename);
-int print_file_action     (GPParams *, const char *filename);
-int print_exif_action     (GPParams *, const char *filename);
-int print_info_action     (GPParams *, const char *filename);
-int save_file_action      (GPParams *, const char *filename);
-int save_thumbnail_action (GPParams *, const char *filename);
-int save_raw_action       (GPParams *, const char *filename);
-int save_audio_action     (GPParams *, const char *filename);
-int save_all_audio_action (GPParams *, const char *filename);
-int save_exif_action      (GPParams *, const char *filename);
-int save_meta_action      (GPParams *, const char *filename);
-int delete_file_action    (GPParams *, const char *filename);
+typedef int FileAction    (GPParams *, const char *folder, const char *filename);
+int print_file_action     (GPParams *, const char *folder, const char *filename);
+int print_exif_action     (GPParams *, const char *folder, const char *filename);
+int print_info_action     (GPParams *, const char *folder, const char *filename);
+int save_file_action      (GPParams *, const char *folder, const char *filename);
+int save_thumbnail_action (GPParams *, const char *folder, const char *filename);
+int save_raw_action       (GPParams *, const char *folder, const char *filename);
+int save_audio_action     (GPParams *, const char *folder, const char *filename);
+int save_all_audio_action (GPParams *, const char *folder, const char *filename);
+int save_exif_action      (GPParams *, const char *folder, const char *filename);
+int save_meta_action      (GPParams *, const char *folder, const char *filename);
+int delete_file_action    (GPParams *, const char *folder, const char *filename);
 
 /* Folder actions */
 typedef int FolderAction  (GPParams *);
@@ -61,7 +77,7 @@ int action_camera_upload_metadata (GPParams *, const char *folder,
 				   const char *path);
 int action_camera_capture_preview (GPParams *);
 int action_camera_capture_movie (GPParams *, const char *arg);
-int action_camera_wait_event (GPParams *,int downloadflag, int count);
+int action_camera_wait_event (GPParams *, enum download_type dt, const char*args);
 
 /* Other actions */
 int list_cameras_action    (GPParams *);
