@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #ifndef __GP_PARAMS_H__
@@ -33,7 +33,8 @@ typedef enum {
 	FLAGS_STDOUT		= 1 << 4,
 	FLAGS_STDOUT_SIZE	= 1 << 5,
 	FLAGS_NEW		= 1 << 6,
-	FLAGS_RESET_CAPTURE_INTERVAL = 1 << 7
+	FLAGS_RESET_CAPTURE_INTERVAL = 1 << 7,
+	FLAGS_KEEP 		= 1 << 8
 } Flags;
 
 typedef enum {
